@@ -52,18 +52,19 @@ using std::endl;
 #include <sidplayfp/SidInfo.h>
 #include <sidplayfp/SidTuneInfo.h>
 
-#ifdef HAVE_CXX11
-#  include <unordered_map>
-    typedef std::unordered_map<std::string, double> filter_map_t;
-    typedef std::unordered_map<std::string, double>::const_iterator filter_map_iter_t;
-#else
-#  include <map>
-    typedef std::map<std::string, double> filter_map_t;
-    typedef std::map<std::string, double>::const_iterator filter_map_iter_t;
-#endif
+
+#include <unordered_map>
+
+using filter_map_t = std::unordered_map<std::string, double>;
+using filter_map_iter_t = std::unordered_map<std::string, double>::const_iterator;
+
 
 // Previous song select timeout (4 secs)
-#define SID2_PREV_SONG_TIMEOUT 4000
+constexpr uint_least32_t SID2_PREV_SONG_TIMEOUT = 4000;
+
+
+const char* ERR_NOT_ENOUGH_MEMORY = "ERROR: Not enough memory.";
+
 
 #ifdef HAVE_SIDPLAYFP_BUILDERS_RESIDFP_H
 #  include <sidplayfp/builders/residfp.h>
@@ -330,6 +331,9 @@ ConsolePlayer::ConsolePlayer (const char * const name) :
     m_outfile(nullptr),
     m_filename(""),
     m_fcurve(-1.0),
+#ifdef FEAT_FILTER_RANGE
+    m_frange(-1.0),
+#endif
     m_quietLevel(0),
     songlengthDB(sldb_t::NONE),
     m_cpudebug(false),
@@ -530,7 +534,7 @@ bool ConsolePlayer::createOutput (output_t driver, const SidTuneInfo *tuneInfo)
     if (!m_driver.device)
     {
         m_driver.device = &m_driver.null;
-        displayError (errnum_t::NOT_ENOUGH_MEMORY);
+        displayError (ERR_NOT_ENOUGH_MEMORY);
         return false;
     }
 
@@ -621,6 +625,11 @@ bool ConsolePlayer::createSidEmu (SIDEMUS emu, const SidTuneInfo *tuneInfo)
                     cerr << "Recommended filter range: " << frange << endl;
             }
 
+            if (m_frange >= 0.0)
+            {
+                frange = m_frange;
+            }
+
             if ((frange < 0.0) || (frange > 1.0))
             {
                 cerr << "Invalid 6581 filter range: " << frange << endl;
@@ -743,15 +752,17 @@ bool ConsolePlayer::createSidEmu (SIDEMUS emu, const SidTuneInfo *tuneInfo)
     {
         if (emu > EMU_DEFAULT)
         {   // No sid emulation?
-            displayError (errnum_t::NOT_ENOUGH_MEMORY);
+            displayError (ERR_NOT_ENOUGH_MEMORY);
             return false;
         }
     }
 
+#ifndef FEAT_FILTER_DISABLE
     if (m_engCfg.sidEmulation) {
         /* set up SID filter. HardSID just ignores call with def. */
         m_engCfg.sidEmulation->filter(m_filter.enabled);
     }
+#endif
 
     return true;
 
@@ -797,6 +808,12 @@ bool ConsolePlayer::open (void)
         displayError(m_engine.error ());
         return false;
     }
+
+#ifdef FEAT_FILTER_DISABLE
+    m_engine.filter(0, m_filter.enabled);
+    m_engine.filter(1, m_filter.enabled);
+    m_engine.filter(2, m_filter.enabled);
+#endif
 #ifdef FEAT_REGS_DUMP_SID
     if (
             (
@@ -815,15 +832,20 @@ bool ConsolePlayer::open (void)
     m_speed.current   = m_speed.max;
     m_engine.fastForward (100 * m_speed.current);
 
-    m_engine.mute(0, 0, vMute[0]);
-    m_engine.mute(0, 1, vMute[1]);
-    m_engine.mute(0, 2, vMute[2]);
-    m_engine.mute(1, 0, vMute[3]);
-    m_engine.mute(1, 1, vMute[4]);
-    m_engine.mute(1, 2, vMute[5]);
-    m_engine.mute(2, 0, vMute[6]);
-    m_engine.mute(2, 1, vMute[7]);
-    m_engine.mute(2, 2, vMute[8]);
+    m_engine.mute(0, 0, m_mute_channel[0]);
+    m_engine.mute(0, 1, m_mute_channel[1]);
+    m_engine.mute(0, 2, m_mute_channel[2]);
+    m_engine.mute(1, 0, m_mute_channel[3]);
+    m_engine.mute(1, 1, m_mute_channel[4]);
+    m_engine.mute(1, 2, m_mute_channel[5]);
+    m_engine.mute(2, 0, m_mute_channel[6]);
+    m_engine.mute(2, 1, m_mute_channel[7]);
+    m_engine.mute(2, 2, m_mute_channel[8]);
+#ifdef FEAT_SAMPLE_MUTE
+    m_engine.mute(0, 3, m_mute_samples[0]);
+    m_engine.mute(1, 3, m_mute_samples[1]);
+    m_engine.mute(2, 3, m_mute_samples[2]);
+#endif
 
     // As yet we don't have a required songlength
     // so try the songlength database or keep the default
@@ -1148,53 +1170,72 @@ void ConsolePlayer::decodeKeys ()
         break;
 
         case A_TOGGLE_VOICE1:
-            vMute[0] = !vMute[0];
-            m_engine.mute(0, 0, vMute[0]);
+            m_mute_channel.flip(0);
+            m_engine.mute(0, 0, m_mute_channel[0]);
         break;
 
         case A_TOGGLE_VOICE2:
-            vMute[1] = !vMute[1];
-            m_engine.mute(0, 1, vMute[1]);
+            m_mute_channel.flip(1);
+            m_engine.mute(0, 1, m_mute_channel[1]);
         break;
 
         case A_TOGGLE_VOICE3:
-            vMute[2] = !vMute[2];
-            m_engine.mute(0, 2, vMute[2]);
+            m_mute_channel.flip(2);
+            m_engine.mute(0, 2, m_mute_channel[2]);
         break;
 
         case A_TOGGLE_VOICE4:
-            vMute[3] = !vMute[3];
-            m_engine.mute(1, 0, vMute[3]);
+            m_mute_channel.flip(3);
+            m_engine.mute(1, 0, m_mute_channel[3]);
         break;
 
         case A_TOGGLE_VOICE5:
-            vMute[4] = !vMute[4];
-            m_engine.mute(1, 1, vMute[4]);
+            m_mute_channel.flip(4);
+            m_engine.mute(1, 1, m_mute_channel[4]);
         break;
 
         case A_TOGGLE_VOICE6:
-            vMute[5] = !vMute[5];
-            m_engine.mute(1, 2, vMute[5]);
+            m_mute_channel.flip(5);
+            m_engine.mute(1, 2, m_mute_channel[5]);
         break;
 
         case A_TOGGLE_VOICE7:
-            vMute[6] = !vMute[6];
-            m_engine.mute(2, 0, vMute[6]);
+            m_mute_channel.flip(6);
+            m_engine.mute(2, 0, m_mute_channel[6]);
         break;
 
         case A_TOGGLE_VOICE8:
-            vMute[7] = !vMute[7];
-            m_engine.mute(2, 1, vMute[7]);
+            m_mute_channel.flip(7);
+            m_engine.mute(2, 1, m_mute_channel[7]);
         break;
 
         case A_TOGGLE_VOICE9:
-            vMute[8] = !vMute[8];
-            m_engine.mute(2, 2, vMute[8]);
+            m_mute_channel.flip(8);
+            m_engine.mute(2, 2, m_mute_channel[8]);
         break;
-
+#ifdef FEAT_SAMPLE_MUTE
+        case A_TOGGLE_SAMPLE1:
+            m_mute_samples.flip(0);
+            m_engine.mute(0, 3, m_mute_samples[0]);
+        break;
+        case A_TOGGLE_SAMPLE2:
+            m_mute_samples.flip(1);
+            m_engine.mute(1, 3, m_mute_samples[1]);
+        break;
+        case A_TOGGLE_SAMPLE3:
+            m_mute_samples.flip(2);
+            m_engine.mute(2, 3, m_mute_samples[2]);
+        break;
+#endif
         case A_TOGGLE_FILTER:
             m_filter.enabled = !m_filter.enabled;
+#ifdef FEAT_FILTER_DISABLE
+            m_engine.filter(0, m_filter.enabled);
+            m_engine.filter(1, m_filter.enabled);
+            m_engine.filter(2, m_filter.enabled);
+#else
             m_engCfg.sidEmulation->filter(m_filter.enabled);
+#endif
         break;
 
         case A_QUIT:
